package com.example.install_plugin;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public class InstallPlugin implements FlutterPlugin, MethodCallHandler {

  private MethodChannel channel;
  private Context mContext;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    mContext = flutterPluginBinding.getApplicationContext();
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "install_plugin");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if ("getPlatformVersion".equals(call.method)) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }else if("installApk".equals(call.method)){
      String filePath = call.argument("filePath");
      String appId = call.argument("appId");
      try {
        installApk(filePath, appId);
        result.success("Success");;
      } catch (Throwable e) {
        result.error(e.getClass().getName(), e.getMessage(), e.getCause());
      }
    } else {
      result.notImplemented();
    }
  }

  void installApk(String filePath,String appId) throws Exception{
    if (filePath == null) throw new NullPointerException("fillPath is null!");
    if(mContext == null) throw new NullPointerException("context is null!");
    File file = new File(filePath);
    if (!file.exists()) throw new FileNotFoundException("$filePath is not exist! or check permission");
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
      install24(mContext, file, appId);
    } else {
      installBelow24(mContext, file);
    }
  }

  private void install24(Context context,File file,String appId) throws Exception{
    if (appId == null) throw new NullPointerException("appId is null!");
    Intent intent = new Intent(Intent.ACTION_VIEW);
    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
    Uri uri = FileProvider.getUriForFile(context, "$appId.fileProvider.install", file);
    intent.setDataAndType(uri, "application/vnd.android.package-archive");
    context.startActivity(intent);
  }


  private void installBelow24(Context context,File file){
    Intent intent = new Intent(Intent.ACTION_VIEW);
    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    Uri uri = Uri.fromFile(file);
    intent.setDataAndType(uri, "application/vnd.android.package-archive");
    context.startActivity(intent);
  }



  //private fun install24(context: Context?, file: File?, appId: String?) {

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}